import 'dart:convert';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/config/settings.dart';
import 'package:scrobbles/cubit/data_counts_by_hour_cubit.dart';
import 'package:scrobbles/models/counts_by_hour.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/card_content.dart';
import 'package:scrobbles/ui/widgets/charts/counts_by_hour.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class CardCountsByHour extends StatelessWidget {
  const CardCountsByHour({super.key});

  @override
  Widget build(BuildContext context) {
    final int daysCount = Settings.countsByHourDaysCount;

    return BlocProvider<DataCountsByHourCubit>(
      create: (BuildContext context) => DataCountsByHourCubit(),
      child: BlocBuilder<DataCountsByHourCubit, DataCountsByHourState>(
        builder: (BuildContext context, DataCountsByHourState state) {
          return CardContent(
            color: Theme.of(context).colorScheme.surface,
            title: 'counts_by_hour'.tr(
              namedArgs: {
                'daysCount': daysCount.toString(),
              },
            ),
            loader: updateCountsByHour(Settings.countsByHourDaysCount),
            content: ChartCountsByHour(
              chartData: CountsByHourData.fromJson(jsonDecode(state.countsByHour.toString())),
              isLoading: false,
            ),
          );
        },
      ),
    );
  }

  Widget updateCountsByHour(int daysCount) {
    final Widget loading = const Text('⏳');
    final Widget done = const Text('');

    late Future<CountsByHourData> futureCountsByHour =
        ScrobblesApi.fetchCountsByHour(daysCount);

    return BlocProvider<DataCountsByHourCubit>(
      create: (BuildContext context) => DataCountsByHourCubit(),
      child: BlocBuilder<DataCountsByHourCubit, DataCountsByHourState>(
        builder: (BuildContext context, DataCountsByHourState state) {
          return FutureBuilder<CountsByHourData>(
            future: futureCountsByHour,
            builder: (context, snapshot) {
              if (snapshot.hasError) {
                return ShowErrorWidget(message: '${snapshot.error}');
              }

              BlocProvider.of<DataCountsByHourCubit>(context).setValue(snapshot.data);
              return !snapshot.hasData ? loading : done;
            },
          );
        },
      ),
    );
  }
}
