import 'dart:convert';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/cubit/data_statistics_global_cubit.dart';
import 'package:scrobbles/models/statistics_global.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/card_content.dart';
import 'package:scrobbles/ui/widgets/content/statistics_global.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class CardStatisticsGlobal extends StatelessWidget {
  const CardStatisticsGlobal({super.key});

  @override
  Widget build(BuildContext context) {
    return BlocProvider<DataStatisticsGlobalCubit>(
      create: (BuildContext context) => DataStatisticsGlobalCubit(),
      child: BlocBuilder<DataStatisticsGlobalCubit, DataStatisticsGlobalState>(
        builder: (BuildContext context, DataStatisticsGlobalState state) {
          return CardContent(
            color: Theme.of(context).colorScheme.primary,
            title: 'global_statistics'.tr(),
            loader: updateStatisticsGlobal(),
            content: ContentStatisticsGlobal(
              statistics:
                  StatisticsGlobalData.fromJson(jsonDecode(state.statisticsGlobal.toString())),
              isLoading: false,
            ),
          );
        },
      ),
    );
  }

  Widget updateStatisticsGlobal() {
    final Widget loading = const Text('⏳');
    final Widget done = const Text('');

    late Future<StatisticsGlobalData> futureStatisticsGlobal =
        ScrobblesApi.fetchGlobalStatistics();

    return BlocProvider<DataStatisticsGlobalCubit>(
      create: (BuildContext context) => DataStatisticsGlobalCubit(),
      child: BlocBuilder<DataStatisticsGlobalCubit, DataStatisticsGlobalState>(
        builder: (BuildContext context, DataStatisticsGlobalState dataState) {
          return FutureBuilder<StatisticsGlobalData>(
            future: futureStatisticsGlobal,
            builder: (context, snapshot) {
              if (snapshot.hasError) {
                return ShowErrorWidget(message: '${snapshot.error}');
              }

              BlocProvider.of<DataStatisticsGlobalCubit>(context).setValue(snapshot.data);
              return !snapshot.hasData ? loading : done;
            },
          );
        },
      ),
    );
  }
}
