import 'dart:convert';

import 'package:easy_localization/easy_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:scrobbles/config/settings.dart';
import 'package:scrobbles/cubit/data_statistics_recent_cubit.dart';
import 'package:scrobbles/models/statistics_recent.dart';
import 'package:scrobbles/network/scrobbles.dart';
import 'package:scrobbles/ui/widgets/card_content.dart';
import 'package:scrobbles/ui/widgets/content/statistics_recent.dart';
import 'package:scrobbles/ui/widgets/error.dart';

class CardStatisticsRecent extends StatelessWidget {
  const CardStatisticsRecent({super.key});

  @override
  Widget build(BuildContext context) {
    final int daysCount = Settings.statisticsRecentDaysCount;

    // data context
    return BlocProvider<DataStatisticsRecentCubit>(
      create: (BuildContext context) => DataStatisticsRecentCubit(),
      child: BlocBuilder<DataStatisticsRecentCubit, DataStatisticsRecentState>(
        builder: (BuildContext context, DataStatisticsRecentState dataState) {
          return CardContent(
            color: Theme.of(context).colorScheme.primary,
            title: 'recent_statistics'.tr(
              namedArgs: {
                'daysCount': daysCount.toString(),
              },
            ),
            loader: updateStatisticsRecent(Settings.statisticsRecentDaysCount),
            content: ContentStatisticsRecent(
              statistics: StatisticsRecentData.fromJson(
                  jsonDecode(dataState.statisticsRecent.toString())),
              isLoading: false,
            ),
          );
        },
      ),
    );
  }

  Widget updateStatisticsRecent(int daysCount) {
    final Widget loading = const Text('⏳');
    final Widget done = const Text('');

    late Future<StatisticsRecentData> futureStatisticsRecent =
        ScrobblesApi.fetchRecentStatistics(daysCount);

    return BlocProvider<DataStatisticsRecentCubit>(
      create: (BuildContext context) => DataStatisticsRecentCubit(),
      child: BlocBuilder<DataStatisticsRecentCubit, DataStatisticsRecentState>(
        builder: (BuildContext context, DataStatisticsRecentState state) {
          return FutureBuilder<StatisticsRecentData>(
            future: futureStatisticsRecent,
            builder: (context, snapshot) {
              if (snapshot.hasError) {
                return ShowErrorWidget(message: '${snapshot.error}');
              }

              BlocProvider.of<DataStatisticsRecentCubit>(context).setValue(snapshot.data);
              return !snapshot.hasData ? loading : done;
            },
          );
        },
      ),
    );
  }
}
